package handlers

import (
	"fmt"
	"github.com/gofiber/fiber/v2"
	"lmika.dev/lmika/hugo-crm/models"
	"lmika.dev/lmika/hugo-crm/services/posts"
)

type Post struct {
	Post *posts.Service
}

func (h *Post) Posts() fiber.Handler {
	return func(c *fiber.Ctx) error {
		site := GetSite(c)

		posts, err := h.Post.ListPostOfSite(c.UserContext(), site)
		if err != nil {
			return err
		}

		return c.Render("posts/index", fiber.Map{
			"site":  site,
			"posts": posts,
		}, "layouts/site")
	}
}

func (h *Post) New() fiber.Handler {
	return func(c *fiber.Ctx) error {
		site := GetSite(c)

		return c.Render("posts/new", fiber.Map{
			"site": site,
			"post": models.Post{},
		}, "layouts/site")
	}
}

func (h *Post) Create() fiber.Handler {
	type Req struct {
		Title string `json:"title" form:"title"`
		Body  string `json:"body" form:"body"`
	}

	return func(c *fiber.Ctx) error {
		site := GetSite(c)

		var req Req
		if err := c.BodyParser(&req); err != nil {
			return err
		}

		_, err := h.Post.Create(c.UserContext(), site, posts.NewPost{
			Title: req.Title,
			Body:  req.Body,
		})
		if err != nil {
			return err
		}

		return c.Redirect(fmt.Sprintf("/sites/%v/posts", site.ID))
	}
}

func (h *Post) Edit() fiber.Handler {
	return func(c *fiber.Ctx) error {
		site := GetSite(c)

		postID, err := c.ParamsInt("postId")
		if err != nil {
			return err
		}

		post, err := h.Post.GetPost(c.UserContext(), postID)
		if err != nil {
			return err
		} else if post.SiteID != site.ID {
			return fmt.Errorf("post id %v not equal to site id %v", postID, site.ID)
		}

		return c.Render("posts/new", fiber.Map{
			"site": site,
			"post": post,
		}, "layouts/site")
	}
}

func (h *Post) Update() fiber.Handler {
	type Req struct {
		Title string `json:"title" form:"title"`
		Body  string `json:"body" form:"body"`
	}

	return func(c *fiber.Ctx) error {
		site := GetSite(c)

		postID, err := c.ParamsInt("postId")
		if err != nil {
			return err
		}

		var req Req
		if err := c.BodyParser(&req); err != nil {
			return err
		}

		post, err := h.Post.GetPost(c.UserContext(), postID)
		if err != nil {
			return err
		} else if post.SiteID != site.ID {
			return fmt.Errorf("post id %v not equal to site id %v", postID, site.ID)
		}

		post.Title = req.Title
		post.Body = req.Body

		if err := h.Post.Save(c.UserContext(), site, &post); err != nil {
			return err
		}

		return c.Redirect(fmt.Sprintf("/sites/%v/posts", site.ID))
	}
}
